/* SPDX-License-Identifier: GPL-2.0 */
/*
 *  Kylin security devctl modules
 *  This file contains the Kylin security devctl hooks defination
 *
 *  Authors:  wangkanglong <wangkanglong@kylinos.cn>
 *
 *  Copyright (C) 2024, KylinSoft Corporation. All Rights Reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2,
 *  as published by the Free Software Foundation.
 */

#ifndef _KSAF_DEVCTL_H_
#define _KSAF_DEVCTL_H_

enum ksaf_device_type {
	KSAF_NONE,
	KSAF_USB,
	KSAF_PCI,
	KSAF_SERIAL,
	KSAF_PARALLEL,
	KSAF_IEEE1394,
	KSAF_PCMCIA,
	KSAF_PCIE,
	KSAF_SATA,
	KSAF_HDMI,
	KSAF_DP,
	KSAF_CDROM,
	KSAF_PRINTER,
	KSAF_WIRELESS,
	KSAF_ETHERNET,
	KSAF_BLUETOOTH,
	KSAF_VIDEO,
	KSAF_AUDIO_JACK,
	KSAF_AUDIO_MIC,
	KSAF_AUDIO_SPEAKER,
	KSAF_INFRARED,
	KSAF_MODEM,
	KSAF_MMC,
	KSAF_SCSI_RAID,
	KSAF_FLOPPY_DRIVE,
	KSAF_MULTI_SERIAL_CARD,
	KSAF_HOTSPOT,
	KSAF_SCANNER,
	KSAF_WINCE,
	KSAF_PRINTER_LOCAL,
	KSAF_PRINTER_NET,
	KSAF_DEV_MAX
};

struct net_device;
struct drm_connector;
struct device;
struct path;
struct pci_dev;
struct block_device;
struct usb_device;
struct cdrom_device_info;
struct usb_interface;

#ifdef CONFIG_SECURITY_KSAF

int ksaf_net_device_permmission(struct net_device *ndev);
int ksaf_pci_dev_permission(struct pci_dev *pdev);
int ksaf_drm_connector_permission(struct drm_connector *connector);
int ksaf_device_permission(struct device *device, int dev_type);
void ksaf_path_mount_set_flags(const char *dev_name, unsigned long *flags);
int ksaf_cdrom_permission(struct cdrom_device_info *cdi,
			  struct block_device *bdev,
			  unsigned int cmd, unsigned long arg);

void ksaf_udev_disconnect(struct usb_device *udev);
void ksaf_device_add(struct device *dev);
void ksaf_device_del(struct device *dev);
int ksaf_for_each_usb_dev(int (*fn)(struct device *, void *));
void ksaf_usb_rebind_intf(struct usb_interface *intf);
void ksaf_cdrom_disconnect(struct cdrom_device_info *cdi);
#else
static inline int ksaf_net_device_permmission(struct net_device *ndev)
{
	return 0;
}

static inline int ksaf_pci_dev_permission(struct pci_dev *pdev)
{
	return 0;
}

static inline int ksaf_drm_connector_permission(struct drm_connector *connector)
{
	return 0;
}

static inline int ksaf_device_permission(struct device *device, int dev_type)
{
	return 0;
}

static inline void ksaf_path_mount_set_flags(const char *dev_name,
					     unsigned long *flags)
{}

static inline int ksaf_cdrom_permission(struct cdrom_device_info *cdi,
					struct block_device *bdev,
					unsigned int cmd, unsigned long arg)
{
	return 0;
}

static inline void ksaf_udev_disconnect(struct usb_device *udev)
{}

static inline void ksaf_device_add(struct device *dev) {}

static inline void ksaf_device_del(struct device *dev) {}

static inline void ksaf_cdrom_disconnect(struct cdrom_device_info *cdi) {}

#endif /* CONFIG_SECURITY_KSAF */
#endif /* _KSAF_DEVCTL_H_ */
