/* SPDX-License-Identifier: GPL-2.0 */
/*
 *  Kylin security modules
 *
 *  This file provides interfaces in v10 kysec1.0
 *
 *  Authors:  penghuaihe <penghuaihe@kylinos.cn>
 *
 *  Copyright (C) 2024, KylinSoft Corporation. All Rights Reserved.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2,
 *  as published by the Free Software Foundation.
 */

#ifndef SECURITY_OPERATIONS_INCLUDE_H
#define SECURITY_OPERATIONS_INCLUDE_H

#include <net/sock.h>
#include <net/sctp/sctp.h>
#include <net/xfrm.h>
#include <linux/key.h>
#include <linux/audit.h>
#include <linux/bpf.h>
#include <linux/security.h>
#include <linux/watch_queue.h>
#include <linux/perf_event.h>
#include <linux/io_uring.h>
#include <linux/pci.h>
#include <linux/blkdev.h>
#include <linux/cdrom.h>
#include <linux/usb.h>
#include <drm/drm_connector.h>
#include <linux/msg.h>

#define SECURITY_NAME_MAX	50

struct nameidata;
struct mnt_idmap;

#ifdef CONFIG_SECURITY_KSAF
/*
 * This struct only contains lsm native hooks. If you wanna use ksaf
 * extended hooks, use ksaf_module_lsm_hook_register interface please.
 */
struct security_operations {
	char name[SECURITY_NAME_MAX + 1];

	#define LSM_HOOK(RET, DEFAULT, NAME, ...) RET (*(NAME))(__VA_ARGS__);
	#include "lsm_hook_defs.h"
	#undef LSM_HOOK
};

void security_del_external_ops(struct security_operations *ops);
int security_add_external_ops(struct security_operations *ops);

#endif /* CONFIG_SECURITY_KSAF */

#endif /* !SECURITY_OPERATIONS_INCLUDE_H */
