#!/bin/bash
#
# Copied from hostap/test/hwsim/start.sh

DIR="$( cd "$( dirname "$0" )" && pwd )"
DRIVERDIR="$(dirname "${DIR}")"
radios=1
dyndbg=/sys/kernel/debug/dynamic_debug/control
cpufreq="/sys/devices/system/cpu/cpufreq"

sysctl="hardlockup_panic hung_task_panic "
sysctl+="panic_on_rcu_stall softlockup_panic "
sysctl+="nmi_watchdog soft_watchdog "


function increase_min_freq_scaling
{
	echo "Performance mode: set min freq to $1"

	ls $cpufreq | while read line
	do
		policy=$cpufreq"/"$line
		sudo sh -c "echo $1 > $policy/scaling_min_freq"
	done
}

function pcie_start
{
	echo "Enabling PCI debug message"
	sudo sh -c "echo 'file *drivers/pci/*.c +p' > ${dyndbg}"

	echo "Setting sysctl"
	for i in $sysctl; do
		sudo sh -c "echo 1 > /proc/sys/kernel/$i"
	done
	sudo sh -c "echo 60 > /proc/sys/kernel/panic"
	sudo sh -c "echo 15 > /proc/sys/kernel/panic_print"
	sudo sh -c "sysctl -w net.core.rmem_max=4194304"
	sudo sh -c "sysctl -w net.core.wmem_max=4194304"
	sudo sh -c "sysctl -w net.core.rmem_default=4194304"
	sudo sh -c "sysctl -w net.core.wmem_default=4194304"
	sudo sh -c "sysctl -w net.core.optmem_max=4194304"
	sudo sh -c "sysctl -w net.ipv4.tcp_rmem=\"4096 87380 4194304\""
	sudo sh -c "sysctl -w net.ipv4.tcp_wmem=\"4096 65536 4194304\""
	sudo sh -c "sysctl -w net.ipv4.tcp_low_latency=1"

	if [ $# -eq 1 ]
	then
		increase_min_freq_scaling 2000000
	elif [ $# -eq 2 ]
	then
		increase_min_freq_scaling $2
	fi

	echo "Loading modules "
	if [ -f /proc/modules ]; then
    		sudo modprobe cfg80211 dyndbg=+p
    		sudo modprobe mac80211 dyndbg=+p
    		sudo insmod ${DRIVERDIR}/scm2625-pcie.ko dyndbg=+p rf=scm2020-1 ring_order=8 n_rx=2
	fi

	# Setup dynamic debug
	echo "Setting dynamic debug"

	if [ -d $DIR/dyndbg.d ]; then
    		echo "Setting up dynamic debug"
    		for f in $DIR/dyndbg.d/*; do sudo $f; done
    		sleep 1
	fi

}

function pcie_stop
{
	if grep -q scm2625_pcie /proc/modules 2>/dev/null ; then
    		echo "Unloading modules"
    		sudo rmmod scm2625-pcie
    		##sudo rmmod mac80211
    		##sudo rmmod cfg80211
	fi
}

case $1 in
	start)
		pcie_stop
		if [ $# -eq 2 ]
		then
			pcie_start $2
		elif [ $# -eq 3 ]
		then
			pcie_start $2 $3
		else
			pcie_start
		fi

		#$DIR/trace.sh start
		;;
	stop)
		#$DIR/trace.sh stop
		pcie_stop
		;;
esac
