#!/bin/bash
#
# Copied from hostap/test/hwsim/start.sh
DIR="$( cd "$( dirname "$0" )" && pwd )"
DRIVERDIR="$(dirname "${DIR}")"
radios=1
dyndbg=/sys/kernel/debug/dynamic_debug/control
DRV_INSTALL_DIR=/lib/modules/$(uname -r)/extra
MODPROB_BLACKLIST=/etc/modprobe.d
fw_lib_path=/lib/firmware/senscomm
fw_bt_path=${fw_lib_path}/bt
scm2020_fw=scm2020.bin
usbwifi=wise-usb-wifi-A1_ram.bin
usbwifi_usbbt=wise-usb-wifi-bt-A1_ram.bin
ftm_usbwifi_bt=wise-usb-wifi-bt-ftm-A1_ram.bin
pciewifi=wise-pcie-wifi-A1_ram.bin
pciewifi_usbbt=wise-pcie-wifi-usb-bt-A1_ram.bin
sysctl="hardlockup_panic hung_task_panic "
sysctl+="panic_on_rcu_stall softlockup_panic "
sysctl+="nmi_watchdog soft_watchdog "
ANT_SEL=
BT_DEV="/dev/ttyUSB0"
FTM_EN=0

function install_pcie_drv
{
	if [ $BT == USB ]
	then
		sudo cp ${DRIVERDIR}/firmware/${pciewifi_usbbt}  ${fw_lib_path}/wise-pcie.bin
		sudo cp ${DRIVERDIR}/scm-btusb.ko $DRV_INSTALL_DIR
	else
		sudo cp ${DRIVERDIR}/firmware/${pciewifi}  ${fw_lib_path}/wise-pcie.bin
	fi
	sudo cp ${DRIVERDIR}/scm2625-pcie.ko $DRV_INSTALL_DIR
}

function install_usb_drv
{
	if [ $BT == USB ]
	then
		sudo cp ${DRIVERDIR}/firmware/${usbwifi_usbbt}  ${fw_lib_path}/wise-usb.bin
		sudo cp ${DRIVERDIR}/scm-btusb.ko $DRV_INSTALL_DIR
	else
		sudo cp ${DRIVERDIR}/firmware/${usbwifi}  ${fw_lib_path}/wise-usb.bin
	fi

	sudo cp ${DRIVERDIR}/scm2625-usb.ko $DRV_INSTALL_DIR
	if [ -d "$MODPROB_BLACKLIST" ]; then
		sudo touch ${MODPROB_BLACKLIST}/scm_blacklist.conf
		sudo sh -c "sudo echo 'blacklist cdc_acm' > ${MODPROB_BLACKLIST}/scm_blacklist.conf"
	fi
}

function drv_install
{
	if [ ! -d "$DRV_INSTALL_DIR" ]; then
		sudo mkdir "$DRV_INSTALL_DIR"
	fi

	if [ ! -d "$fw_lib_path" ]; then
		sudo mkdir "$fw_lib_path"
	fi
	sudo cp ${DRIVERDIR}/firmware/${scm2020_fw}  ${fw_lib_path}/

	case $WLAN in
		PCIE)
			install_pcie_drv
			;;

		USB)
			install_usb_drv
			;;
	esac

	sudo depmod -aA
	echo "$WLAN WLAN + $BT BT install to $DRV_INSTALL_DIR success"
}

function drv_uninstall
{
	if [ ! -d "$DRV_INSTALL_DIR" ]; then
		return
	fi

	case $WLAN in
		PCIE)
			if [ $BT == USB ]
			then
				sudo rm $DRV_INSTALL_DIR/scm-btusb.ko
			fi
			sudo rm $DRV_INSTALL_DIR/scm2625-pcie.ko
			;;

		USB)
			if [ $BT == USB ]
			then
				sudo rm $DRV_INSTALL_DIR/scm-btusb.ko
			fi
			sudo rm $DRV_INSTALL_DIR/scm2625-usb.ko
			;;
	esac

	sudo depmod
	echo "$WLAN WLAN + $BT BT remove from $DRV_INSTALL_DIR success"
}

function load_drv
{
	sudo cp ${DRIVERDIR}/firmware/${scm2020_fw}  ${fw_lib_path}/
	EXT_PARMS="dev_ant=$(($ANT_SEL-1))"

	case $WLAN in
		PCIE)
			if [ $BT == USB ]
			then
				sudo cp ${DRIVERDIR}/firmware/${pciewifi_usbbt}  ${fw_lib_path}/wise-pcie.bin
				sudo modprobe bluetooth
				sudo insmod ${DRIVERDIR}/scm-btusb.ko dyndbg=+p
			else
				sudo cp ${DRIVERDIR}/firmware/${pciewifi}  ${fw_lib_path}/wise-pcie.bin
			fi

			sudo insmod ${DRIVERDIR}/scm2625-pcie.ko dyndbg=+p $EXT_PARMS
			;;

		USB)
			if [ $BT == USB ]
			then
				if [ $FTM_EN == 1 ]
				then
					sudo cp ${DRIVERDIR}/firmware/${ftm_usbwifi_bt}  ${fw_lib_path}/wise-usb.bin
				else
					sudo cp ${DRIVERDIR}/firmware/${usbwifi_usbbt}  ${fw_lib_path}/wise-usb.bin
				fi

				sudo modprobe bluetooth
				sudo insmod ${DRIVERDIR}/scm-btusb.ko dyndbg=+p
			else
				if [ $FTM_EN == 1 ]
				then
					sudo cp ${DRIVERDIR}/firmware/${ftm_usbwifi_bt}  ${fw_lib_path}/wise-usb.bin
				else
					sudo cp ${DRIVERDIR}/firmware/${usbwifi}  ${fw_lib_path}/wise-usb.bin
				fi
			fi

			if grep -q cdc_acm /proc/modules 2>/dev/null ; then
				echo "Unloading cdc_acm modules"
				sudo rmmod cdc_acm
			fi
			sudo insmod ${DRIVERDIR}/scm2625-usb.ko dyndbg=+p $EXT_PARMS

			;;
	esac

	sudo sh -c "echo \"module xhci_hcd -p\" > /sys/kernel/debug/dynamic_debug/control"

	if [ $BT == UART ]; then
		echo "${BT_DEV}" > "${fw_bt_path}/port"
	fi
}

function rm_fw
{
	if [ -e ${fw_lib_path}/wise-pcie.bin ]; then
		echo "remove wise-pcie.bin!"
		sudo rm ${fw_lib_path}/wise-pcie.bin
	fi
	if [ -e ${fw_lib_path}/wise-usb.bin ]; then
		echo "remove wise-usb.bin!"
		sudo rm ${fw_lib_path}/wise-usb.bin
	fi
}

function drv_start
{
	echo "Enabling PCI debug message"
	sudo sh -c "echo 'file *drivers/pci/*.c +p' > ${dyndbg}"

	echo "Enabling USB debug message"
	sudo sh -c "echo 'file *drivers/usb/*.c +p' > ${dyndbg}"

	echo "Setting sysctl"
	for i in $sysctl; do
		sudo sh -c "echo 1 > /proc/sys/kernel/$i"
	done

	sudo sh -c "echo 60 > /proc/sys/kernel/panic"
	sudo sh -c "echo 15 > /proc/sys/kernel/panic_print"
	sudo sh -c "sysctl -w net.core.rmem_max=4194304"
	sudo sh -c "sysctl -w net.core.wmem_max=4194304"
	sudo sh -c "sysctl -w net.core.rmem_default=4194304"
	sudo sh -c "sysctl -w net.core.wmem_default=4194304"
	sudo sh -c "sysctl -w net.core.optmem_max=4194304"
	sudo sh -c "sysctl -w net.ipv4.tcp_rmem=\"4096 87380 4194304\""
	sudo sh -c "sysctl -w net.ipv4.tcp_wmem=\"4096 65536 4194304\""
	sudo sh -c "sysctl -w net.ipv4.tcp_low_latency=1"

	echo "Loading modules "
	if [ -f /proc/modules ]; then
		sudo modprobe cfg80211 dyndbg=+p
		sudo modprobe mac80211 dyndbg=+p
	fi

	load_drv

	# Setup dynamic debug
	echo "Setting dynamic debug"

	if [ -d $DIR/dyndbg.d ]; then
		echo "Setting up dynamic debug"
			for f in $DIR/dyndbg.d/*; do sudo $f; done
			sleep 1
	fi
}

function drv_stop
{
	if grep -q scm2625_pcie /proc/modules 2>/dev/null ; then
		echo "Unloading Wireless modules"
		sudo rmmod scm2625-pcie
	fi
	if grep -q scm2625_usb /proc/modules 2>/dev/null ; then
		echo "Unloading Wireless modules"
		sudo rmmod scm2625-usb
	fi
	if grep -q scm_btusb /proc/modules 2>/dev/null ; then
		echo "Unloading Bluetooth modules"
		sudo rmmod scm_btusb
	fi

	#rm_fw
}

combination=3

function usage
{
	echo "Usage"
	echo "	./run.sh [operation] [combination] [ftm enable]"
	echo ""
	echo "operation"
	echo "	start				;load driver"
	echo "	stop				;unload driver"
	echo "	install				;install driver"
	echo "	remove				;uninstall driver"
	echo "	help				;show help info"
	echo "combination"
	echo "	1				;WIFI:PCIE, BT:NONE/UART + 2-ANT"
	echo "	2				;WIFI:PCIE, BT:NONE/UART + 3-ANT"
	echo "	3				;WIFI:PCIE, BT:USB + 2-ANT"
	echo "	4				;WIFI:PCIE, BT:USB + 3-ANT"
	echo "	5				;WIFI:USB, BT:NONE/UART + 2-ANT"
	echo "	6				;WIFI:USB, BT:NONE/UART + 3-ANT"
	echo "	7				;WIFI:USB, BT:USB + 2-ANT"
	echo "	8				;WIFI:USB, BT:USB + 3-ANT"
	echo "	default combination		;$combination"
	echo "ftm enable"
	echo "	ftm				;enable FTM mode, default 0"
}

if [ $# -gt 0 ]
then
	if [ $# -ge 2 ]
	then
		combination=$2
	fi

	case $combination in
	1)
		echo "WIFI:PCIE, BT:NONE/UART + 2-ANT"
		WLAN=PCIE
		BT=NONE
		ANT_SEL=2
		;;
	2)
		echo "WIFI:PCIE, BT:NONE/UART + 3-ANT"
		WLAN=PCIE
		BT=NONE
		ANT_SEL=3
		;;
	3)
		echo "WIFI:PCIE, BT:USB + 2-ANT"
		WLAN=PCIE
		BT=USB
		ANT_SEL=2
		;;
	4)
		echo "WIFI:PCIE, BT:USB + 3-ANT"
		WLAN=PCIE
		BT=USB
		ANT_SEL=3
		;;
	5)
		echo "WIFI:USB, BT:NONE/UART + 2-ANT"
		WLAN=USB
		BT=NONE
		ANT_SEL=2
		;;
	6)
		echo "WIFI:USB, BT:NONE/UART + 3-ANT"
		WLAN=USB
		BT=NONE
		ANT_SEL=3
		;;
	7)
		echo "WIFI:USB, BT:USB + 2-ANT"
		WLAN=USB
		BT=USB
		ANT_SEL=2
		;;
	8)
		echo "WIFI:USB, BT:USB + 3-ANT"
		WLAN=USB
		BT=USB
		ANT_SEL=3
		;;
	esac

	if [ $# -ge 3 ] && [ "ftm" == $3 ]
	then
		FTM_EN=1
	fi

	if [ "$1" == "stop" ]
	then
		drv_stop
	elif [ "$1" == "start" ]
	then
		drv_stop
		drv_start
	elif [ "$1" == "install" ]
	then
		drv_install
	elif [ "$1" == "remove" ]
	then
		drv_uninstall
	else
		usage
	fi
else
	usage
fi
