#!/bin/bash
#
# Copied from hostap/test/hwsim/start.sh

DIR="$( cd "$( dirname "$0" )" && pwd )"
DRIVERDIR="$(dirname "${DIR}")"
radios=1
dyndbg=/sys/kernel/debug/dynamic_debug/control
udev_path=/lib/udev/rules.d
rules_file=99-scm2020.rules
download=adownload-tiny.native
firmware=wise-a0-usb-mem.bin
#firmware=wise-a0-usb-flash.bin
sysctl="hardlockup_panic hung_task_panic "
sysctl+="panic_on_rcu_stall softlockup_panic "
sysctl+="nmi_watchdog soft_watchdog "

function usb_install
{
	if [ ! -e ${udev_path}/${rules_file} ]; then
		echo "copy udev rules to system!"
		echo -n "ACTION==\"add|change\", SUBSYSTEM==\"tty\", KERNEL==\"ttyACM[0-9]\", ATTRS{idVendor}==\"2ecc\", ATTRS{idProduct}==\"3004\", MODE:=\"0777\" " >${DRIVERDIR}/scripts/${rules_file}
		echo -n "RUN+=\"${DRIVERDIR}/scripts/${download} -d /dev/\$name -g  ${DRIVERDIR}/firmware/${firmware}\"" >>${DRIVERDIR}/scripts/${rules_file}
		sudo cp ${DRIVERDIR}/scripts/${rules_file}  ${udev_path}/${rules_file}
		sudo service udev restart
	fi
}

function usb_uninstall
{
	if [ -e ${udev_path}/${rules_file} ]; then
		echo "remove udev rules from system!"
		sudo rm ${udev_path}/${rules_file}
		sudo rm ${DRIVERDIR}/scripts/${rules_file}
		sudo service udev restart
	fi
}

function usb_start
{
	echo "Enabling USB debug message"
	sudo sh -c "echo 'file *drivers/usb/*.c +p' > ${dyndbg}"

	echo "Setting sysctl"
	for i in $sysctl; do
		sudo sh -c "echo 1 > /proc/sys/kernel/$i"
	done

	sudo sh -c "echo 60 > /proc/sys/kernel/panic"
	sudo sh -c "echo 15 > /proc/sys/kernel/panic_print"
	sudo sh -c "sysctl -w net.core.rmem_max=4194304"
	sudo sh -c "sysctl -w net.core.wmem_max=4194304"
	sudo sh -c "sysctl -w net.core.rmem_default=4194304"
	sudo sh -c "sysctl -w net.core.wmem_default=4194304"
	sudo sh -c "sysctl -w net.core.optmem_max=4194304"
	sudo sh -c "sysctl -w net.ipv4.tcp_rmem=\"4096 87380 4194304\""
	sudo sh -c "sysctl -w net.ipv4.tcp_wmem=\"4096 65536 4194304\""
	sudo sh -c "sysctl -w net.ipv4.tcp_low_latency=1"

	echo "Loading modules "
	if [ -f /proc/modules ]; then
		sudo modprobe cfg80211 dyndbg=+p
		sudo modprobe mac80211 dyndbg=+p
		sudo insmod ${DRIVERDIR}/scm2625-usb.ko dyndbg=+p rf=scm2020-5 ring_order=8 n_rx=2
	fi
	if [ "$1" == "bt" ]
	then
		sudo modprobe bluetooth
		sudo insmod ${DRIVERDIR}/scm-btusb.ko dyndbg=+p
	fi
	# Setup dynamic debug
	echo "Setting dynamic debug"

	if [ -d $DIR/dyndbg.d ]; then
		echo "Setting up dynamic debug"
			for f in $DIR/dyndbg.d/*; do sudo $f; done
			sleep 1
	fi
	sudo sh -c "echo \"module xhci_hcd -p\" > /sys/kernel/debug/dynamic_debug/control"
}

function usb_stop
{
	if grep -q scm2625_usb /proc/modules 2>/dev/null ; then
		echo "Unloading Wireless modules"
		sudo rmmod scm2625-usb
		##sudo rmmod mac80211
		##sudo rmmod cfg80211
	fi
	if grep -q scm_btusb /proc/modules 2>/dev/null ; then
		echo "Unloading Bluetooth modules"
		sudo rmmod scm_btusb
	fi
	if grep -q scm2625_usb_dl /proc/modules 2>/dev/null ; then
		echo "Unloading Firmware Download modules"
#		sudo rmmod scm2625_usb_dl
	fi
}

case $1 in
	install)
		usb_uninstall
		usb_install
		;;
	start)
		usb_stop
		if [ $# -eq 1 ]
		then
			usb_start
		fi
		if [ $# -eq 2 ]
		then
			if [ "$2" == "bt" ]
			then
				usb_start bt
			fi
			if [ "$2" == "dl_fw" ]
			then
				sudo rmmod cdc_acm
#				sudo insmod ${DRIVERDIR}/scm2625-usb-dl.ko dyndbg=+p
				usb_start
			fi
			if [ "$2" == "dl_fw_bt" ]
			then
				sudo rmmod cdc_acm
#				sudo insmod ${DRIVERDIR}/scm2625-usb-dl.ko dyndbg=+p
				usb_start bt
			fi
		fi
		#$DIR/trace.sh start
		;;
	stop)
		#$DIR/trace.sh stop
		usb_stop
		;;
esac
