#!/bin/bash

##############################################################
##
## 本脚本用于研发黑盒自测，每次代码修改后，修改的新版本部署以后，
## 可通过此运行此脚本，来测试ksaf的基本管控能力，从而确认本次代码
## 修改，是否对基本功能产生影响，增强代码质量。
##
#############################################################

export SRC=$(dirname "$(realpath "$0")")
export WORK_DIR="/ksaf_main_kunit"
export ksaf_selftest_result="PASS"

cd $SRC
source "$SRC"/kunit/lib.sh

export KSAF_STATUS_FILE="/sys/kernel/security/ksaf/status"
export KSAF_STATUS_BAK=$(cat $KSAF_STATUS_FILE)
export KSAF_POLICYINFO_FILE="/sys/kernel/security/ksaf/common/policy_info"

ksaf_change_status()
{
        local ksaf_status=$(cat $KSAF_STATUS_FILE)

        case $1 in
                0|2|4)
                        ;;
                *)
                        print_info "bad ksaf status given: $1 stop test."
                        exit -1;
                        ;;
        esac

        case $ksaf_status in
                0)
                        print_info "ksaf is disabled now! stop test."
                        exit -1;
                        ;;
                2|4)
                        echo $1 > $KSAF_STATUS_FILE
                        ;;
                *)
                        ;;
        esac
}

# $1:result  $2:expect  $3:testcase_name
ksaf_print_result()
{
        if [ "$#" -ne 3 ]; then
                print_info "parameters invalid!"
                exit 1
        fi

        if [ "$2" = "error" ];then
                if [ "$1" = "0" ];then
                        print_fail "$3 测试失败!"
                        ksaf_selftest_result="FAILED"
                else
                        print_success "$3 测试通过!"
                fi
        else
                if [ "$1" = "0" ];then
                        print_success "$3 测试通过!"
                else
                        print_fail "$3 测试失败!"
                        ksaf_selftest_result="FAILED"
                fi
        fi
}


set -e

ksaf_change_status 4
# 备份原策略
rm -rf /etc/ksaf/policy.selftest.bak/*
mkdir -p /etc/ksaf/policy.selftest.bak
mkdir -p /etc/ksaf/policy
mv -f /etc/ksaf/policy/* /etc/ksaf/policy.selftest.bak/
ksaf_change_status 2

# 基本功能自测策略
cat <<EOF > /etc/ksaf/policy/selftest.p
string name = "selftest"

disallow * * "/opt/selftest" file {write}
disallow * * "/opt/echo" file {exec}
disallow * * "/opt/selftest.sh" file {interpret}
disallow * "/opt/ping" * socket {connect}

EOF

# 策略下发
ksaf-main-policy-sync
ksaf_print_result $? normal 策略编译

# 使用enforcing模式


set +e

# 测试内核是否收到策略
policy_matching_lines=$(grep -F "2685a1ca33c33693" "$KSAF_POLICYINFO_FILE")
if [ -n "$policy_matching_lines" ];then
        print_success "策略下发 测试通过！"
else
        print_fail "策略下发测试项测试失败！"
        ksaf_selftest_result="FAILED"
fi

# 只读保护测试项
echo "test" > /opt/selftest
ksaf_print_result $? error 只读保护

# 二进制执行管控
cp -f `which echo` /opt/
/opt/echo "test"
ksaf_print_result $? error 二进制执行管控

# 脚本执行管控
cat <<EOF > /opt/selftest.sh
echo "ksaf selftest shell"
EOF
bash /opt/selftest.sh
ksaf_print_result $? error 脚本执行管控

# 联网管控测试
cp -f `which ping` /opt/
ping -c 1 127.0.0.1
ksaf_print_result $? normal 联网放行
/opt/ping -c 1 127.0.0.1
ksaf_print_result $? error 联网管控

if [ $ksaf_selftest_result = "PASS" ];then
        print_success "#################################"
        print_success "      全部测试项测试通过"
        print_success "#################################"
else
        print_fail "#################################"
        print_fail "      有测试项未通过，测试失败"
        print_fail "#################################"
fi

#恢复环境
ksaf_change_status 4
rm -rf /etc/ksaf/policy/*
mv -f  /etc/ksaf/policy.selftest.bak/* /etc/ksaf/policy/
rm -rf /etc/ksaf/policy.selftest.bak/
ksaf_change_status $KSAF_STATUS_BAK
