#!/bin/bash
# SPDX-License-Identifier: GPL-2.0

TRACEING=/sys/kernel/debug/tracing
TRACE=${TRACEING}/trace
TRACE_PIPE=${TRACEING}/trace_pipe
KPROBE_EVENTS=${TRACEING}/kprobe_events
TRACE_EVENTS=${TRACEING}/events
TRACE_EVENTS_ENABLE=${TRACE_EVENTS}/enable
KPROBES=${TRACE_EVENTS}/kprobes
KPROBE_EVENTS_ENABLE=${KPROBES}/enable
LOGFILE=kprobe.log


ENABLE=1
DISABLE=0

function kprobe_exit() {
    sleep 1
    disable_kprobe_events
    disable_trace_events
    sync
    clean_kprobe_events
    clean_trace
}

function trace_exit() {
    sleep 1
    disable_trace_events
    sync
    clean_trace
}

function check_args_num() {
    if [ ${1} -ne ${2} ];then
        echo "args is error"
        kprobe_exit
        exit
    fi
}

function readto() {
    check_args_num 2 $#
    printf "cat $1 | tee $2\n"
    cat $1 | tee $2
}

function writeto() {
    check_args_num 2 $#
    echo ${1} > ${2}
    printf "echo \"${1}\" > ${2}\n"
}

function appendto() {
    check_args_num 2 $#
    echo ${1} >> ${2}
    printf "echo \"${1}\" >> ${2}\n"
}

function clean_trace() {
    writeto "" "${TRACE}"
}

function clean_kprobe_events() {
    writeto "" "${KPROBE_EVENTS}"
}

function enable_kprobe_event() {
    check_args_num 1 $#
    writeto "${ENABLE}" "${KPROBES}/${1}/enable"
}

function enable_kprobe_events() {
    writeto "${ENABLE}" "${KPROBE_EVENTS_ENABLE}"
}

function disable_kprobe_event() {
    check_args_num 1 $#
    writeto ${DISABLE} ${KPROBES}/${1}/enable
}

function disable_kprobe_events() {
    writeto ${DISABLE} ${KPROBE_EVENTS_ENABLE}
	echo "disable_events"
}

function set_kprobe_event() {
    check_args_num 1 $#
    writeto "${1}" "${KPROBE_EVENTS}"
}

function add_kprobe_event() {
    check_args_num 1 $#
    appendto "${1}" "${KPROBE_EVENTS}"
}

function set_kprobe_event_filter() {
    check_args_num 2 $#
    writeto "${1}" "${KPROBES}/${2}/filter"
}

function get_trace_pipe() {
    if [ $# -eq 1 ];then
        LOGFILE=${1}
    fi
    readto "${TRACE_PIPE}" ${LOGFILE}
}

function get_trace() {
    if [ $# -eq 1 ];then
        LOGFILE=${1}
    fi
    readto "${TRACE_PIPE}" ${LOGFILE}
}

function enable_trace_event() {
    writeto "${ENABLE}" "${TRACE_EVENTS}/${1}/enable"
}

function set_trace_event_filter() {
    check_args_num 2 $#
    writeto "${1}" "${TRACE_EVENTS}/${2}/filter"
}

function enable_trace_events() {
    writeto "${ENABLE}" "${TRACE_EVENTS}/enable"
}

function disable_trace_event() {
    writeto "${DISABLE}" "${TRACE_EVENTS}/${1}/enable"
}

function disable_trace_events() {
    writeto "${DISABLE}" "${TRACE_EVENTS}/enable"
}

# krpbe IP frag
function enable_kprobe_IP() {
    check_args_num 1 $#
    add_kprobe_event "r:${1}_1"' inet_frag_find $retval'
    set_kprobe_event_filter 'arg1 == 0' ${1}_1

    add_kprobe_event "r:${1}_2"' ip_defrag $retval'
    set_kprobe_event_filter 'arg1 != 0xffffff8d && arg1 != 0' ${1}_2
}

function disable_kprobe_IP() {
    check_args_num 1 $#
    disable_kprobe_event "${1}_1"
    disable_kprobe_event "${1}_2"
}
