#!/bin/bash
# SPDX-License-Identifier: GPL-2.0
###
 # @Author: zhubing <zhubing@kylinos.cn>
 # @FilePath: /kysec_testcase/kysec_exectl/test_exectl.sh
 # @Description: shell scripts for kylin security execute control test
 #
 # Copyright (C) 2022-2024 KylinSoft Corporation.
###

###
 # @function: test_exectl_enforcing
 # @description: kysec exectl enforcing test
 # @return {*} 0: success; !0: failed
###
function test_exectl_enforcing {
    setstatus -f exectl enforcing
    start_time=`date +%s`
    file=ls-${start_time}
    cp /bin/ls /tmp/$file
    /tmp/$file
    ret_unknown=$?

    kysec_set -n exectl -v verified /tmp/$file
    /tmp/$file
    ret_verified=$?

    cp /bin/cat /tmp/$file
    /tmp/$file
    ret_distort=$?

    cp /bin/ls /tmp/$file
    /tmp/$file
    ret_measure=$?
    #exec kysoft label test
    if [ $ret_unknown -ne 0 ] && [ $ret_verified -eq 0 ] && [ $ret_distort -ne 0 ] && [ $ret_measure -eq 0 ]
    then
    	echo "exectl enforcing test ok......"
    	ret_enforcing=0
    	echo `date +%Y%m%d%H%M%S`:test_exectl:test_exectl_enforcing:res ok >>   res_exectl
    else
    	echo "exectl enforcing test failed......"
    	ret_enforcing=-1
	    echo `date +%Y%m%d%H%M%S`:test_exectl:test_exectl_enforcing:res failed >> res_exectl
    fi
    return $ret_enforcing
}

###
 # @function: test_exectl_warning
 # @description: kysec exectl warning test
 # @return {*} 0: success; !0: failed
###
function test_exectl_warning {
    setstatus -f exectl warning
    start_time=`date +%s`
    file=ls-${start_time}
    cp /bin/ls /tmp/$file
    /tmp/$file
    ret_unknown=$?
    end_time=`date +%s`
    let during=${end_time}-${start_time}
    ret_auth=-1
    if [ $ret_unknown -ne 0 ] && [ $during -ge 30 ]
    then
        ret_auth=0
    fi
    kysec_set -n exectl -v verified /tmp/$file
    cp /bin/cat  /tmp/$file
    cp /bin/ls /tmp/$file
    /tmp/$file
    ret_measure=$?
    #exec kysoft label test
    if [ $ret_auth -eq 0 ] && [ $ret_measure -eq 0 ]
    then
    	echo "exectl warning test ok......"
    	ret_warning=0
    	echo `date +%Y%m%d%H%M%S`:test_exectl:test_exectl_warning:res ok >>   res_exectl
    else
    	echo "exectl warning test failed......"
    	ret_warning=-1
    	echo `date +%Y%m%d%H%M%S`:test_exectl:test_exectl_waring:res failed >> res_exectl
    fi

    return $ret_warning
}

###
 # @function: test_exectl_off
 # @description: kysec exectl off test
 # @return {*} 0: success; !0: failed
###
function test_exectl_off {
    setstatus -f exectl off
    start_time=`date +%s`
    file=ls-${start_time}
    cp /bin/ls /tmp/$file
    /tmp/$file
    ret_unknown=$?

    if [ $ret_unknown -eq 0 ]
    then
    	echo "exectl off test ok......"
    	ret_warning=0
    	echo `date +%Y%m%d%H%M%S`:test_exectl:test_exectl_off:res ok >>   res_exectl
    else
    	echo "exectl warning test failed......"
    	ret_warning=-1
    	echo `date +%Y%m%d%H%M%S`:test_exectl:test_exectl_off:res failed >> res_exectl
    fi

    return $ret_off
}

rm -rf res_exectl
test_exectl_enforcing
res_test_exectl_enforcing=$?
sleep 2
test_exectl_warning
res_test_exectl_warning=$?
sleep 2
test_exectl_off
res_test_exectl_off=$?

if [ $res_test_exectl_enforcing -eq 0 ] && [ $res_test_exectl_warning -eq 0 ] && [ $res_test_exectl_off -eq 0 ]
then
    echo `date +%Y%m%d%H%M%S`:test_exectl:testcase:res ok >> res_exectl
else
    echo `date +%Y%m%d%H%M%S`:test_exectl:testcase:res failed >> res_exectl
fi
