#!/bin/bash
# SPDX-License-Identifier: GPL-2.0
###
 # @Author: zhubing <zhubing@kylinos.cn>
 # @FilePath: /kysec_testcase/kysec_exectl/test_kysec_exectl.sh
 # @Description: shell scripts for kylin security execute control test
 #
 # Copyright (C) 2022-2024 KylinSoft Corporation.
###

###
 # @function: test_kysec_exectl_enforcing
 # @description: kysec exectl enforcing test
 # @return {*} 0: success; !0: failed
###
function test_kysec_exectl_enforcing {
    setstatus -f exectl enforcing
    start_time=`date +%s`
    file=ls-${start_time}
    cp /bin/ls /tmp/$file
    /tmp/$file
    ret_unknown=$?

    kysec_set -n exectl -v verified /tmp/$file
    /tmp/$file
    ret_verified=$?

    cp /bin/cat /tmp/$file
    /tmp/$file
    ret_distort=$?

    cp /bin/ls /tmp/$file
    /tmp/$file
    ret_measure=$?
    #exec kysoft label test
    if [ $ret_unknown -ne 0 ] && [ $ret_verified -eq 0 ] && [ $ret_distort -ne 0 ] && [ $ret_measure -eq 0 ]
    then
    	echo "exectl enforcing test ok......"
    	ret_enforcing=0
    	echo `date +%Y%m%d%H%M%S`:test_kysec_exectl:test_kysec_exectl_enforcing:res ok >>   res_kysec_exectl
    else
    	echo "exectl enforcing test failed......"
    	ret_enforcing=-1
    	echo `date +%Y%m%d%H%M%S`:test_kysec_exectl:test_kysec_exectl_enforcing:res failed >> res_kysec_exectl
    fi
    return $ret_enforcing
}

###
 # @function: test_kysec_exectl_measure
 # @description: kysec exectl measure test
 # @return {*} 0: success; !0: failed
###
function test_kysec_exectl_measure {
    #警告模式下度量
    setstatus -f exectl warning
    start_time=`date +%s`
    file=ls-${start_time}
    cp /bin/ls /tmp/$file
    kysec_set -n exectl -v verified /tmp/$file
    cp /bin/cat  /tmp/$file
    cp /bin/ls /tmp/$file
    /tmp/$file
    ret_warning=$?
    file_warning=`kysec_get /tmp/$file  |awk -F ":" '{print $4}'`
    #exec kysoft label test
    sleep 1

    #强制模式下度量
    setstatus -f exectl enforcing
    start_time=`date +%s`
    file=ls-${start_time}
    cp /bin/ls /tmp/$file
    kysec_set -n exectl -v verified /tmp/$file
    cp /bin/cat  /tmp/$file
    cp /bin/ls /tmp/$file
    /tmp/$file
    ret_enforcing=$?
    file_enforcing=`kysec_get /tmp/$file  |awk -F ":" '{print $4}'`

    if  [ $ret_warning -eq 0 ] && [ $file_warning == "verified" ] && [ $ret_enforcing -eq 0 ] \
	    && [ $file_enforcing == "verified" ]
    then
        echo "exectl measure test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_exectl:test_kysec_exectl_measure:res ok >>   res_kysec_exectl
    else
        echo "exectl measure test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_exectl:test_kysec_exectl_measure:res failed >> res_kysec_exectl
    fi


    return $ret
}

###
 # @function: test_kysec_exectl_off
 # @description: kysec exectl off test
 # @return {*} 0: success; !0: failed
###
function test_kysec_exectl_off {
    setstatus -f exectl off
    start_time=`date +%s`
    file=ls-${start_time}
    cp /bin/ls /tmp/$file
    /tmp/$file
    ret_unknown=$?

    if [ $ret_unknown -eq 0 ]
    then
    	echo "exectl off test ok......"
    	ret_warning=0
    	echo `date +%Y%m%d%H%M%S`:test_kysec_exectl:test_kysec_exectl_off:res ok >>   res_kysec_exectl
    else
    	echo "exectl warning test failed......"
    	ret_warning=-1
    	echo `date +%Y%m%d%H%M%S`:test_kysec_exectl:test_kysec_exectl_off:res failed >> res_kysec_exectl
    fi

    return $ret_off
}

###
 # @function: test_kysec_exectl_parent_invokekysoft
 # @return {*} 0: success; !0: failed
###
function test_kysec_exectl_parent_invokekysoft {
    setstatus -f exectl enforcing

    kysec_set -n exectl -v parent ./parent_invoke_kysoft.sh
    kysec_set -n exectl -v kysoft ./kysoft_file.sh

    start_time=`date +%s`
    file=ls-${start_time}
    ./parent_invoke_kysoft.sh $file
    file_lable=`kysec_get /tmp/$file  |awk -F ":" '{print $4}'`

    if [ "$file_lable" == "verified" ]
    then
        ret_parent_invoke=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_exectl:test_kysec_exectl_parent_invokekysoft:res ok >>   res_kysec_exectl
    else
        ret_parent_invoke=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_exectl:test_kysec_exectl_parent_invokekysoft:res failed >> res_kysec_exectl
    fi

    return $ret_parent_invoke
}

###
 # @function: test_kysec_exectl_kysoft
 # @return {*} 0: success; !0: failed
###
function test_kysec_exectl_kysoft {
    setstatus -f exectl enforcing

    kysec_set -n exectl -v kysoft ./kysoft_file.sh

    start_time=`date +%s`
    file=ls-${start_time}
    ./kysoft_file.sh $file
    file_lable=`kysec_get /tmp/$file  |awk -F ":" '{print $4}'`

    if [ "$file_lable" == "verified" ]
    then
        ret_kysoft=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_exectl:test_kysec_exectl_kysoft:res ok >>   res_kysec_exectl
    else
        ret_kysoft=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_exectl:test_kysec_exectl_kysoft:res failed >> res_kysec_exectl
    fi

    return $ret_kysoft
}


rm -rf res_kysec_exectl
kysec_status=`cat /sys/kernel/security/kysec/status`
if [ $kysec_status -eq 2 ] ; then
    echo "kysec is enable"
elif [ $kysec_status -eq 4 ] ; then
    echo "kysec status is softmode"
    setstatus enable
else
    echo `date +%Y%m%d%H%M%S`:kysec_exectl:testcase:res failed >> res_kysec_exectl
    echo `date +%Y%m%d%H%M%S`:"kysec staus is disable" >> res_kysec_exectl
    exit
fi


test_kysec_exectl_enforcing
res_test_kysec_exectl_enforcing=$?
sleep 1
test_kysec_exectl_measure
res_test_kysec_exectl_measure=$?
sleep 1
test_kysec_exectl_off
res_test_kysec_exectl_off=$?
sleep 1
test_kysec_exectl_parent_invokekysoft
res_parent_invokekysoft=$?
sleep 1
test_kysec_exectl_kysoft
res_kysoft=$?

if [ $res_test_kysec_exectl_enforcing -eq 0 ] && [ $res_test_kysec_exectl_measure -eq 0 ] \
	&& [ $res_test_kysec_exectl_off -eq 0 ]  && [ $res_parent_invokekysoft -eq 0 ] && [ $res_kysoft -eq 0 ]
then
    echo `date +%Y%m%d%H%M%S`:test_kysec_exectl:testcase:res ok >> res_kysec_exectl
else
    echo `date +%Y%m%d%H%M%S`:test_kysec_exectl:testcase:res failed >> res_kysec_exectl
fi

setstatus softmode
