#!/bin/bash
# SPDX-License-Identifier: GPL-2.0
###
 # @Author: zhubing <zhubing@kylinos.cn>
 # @FilePath: /kysec_testcase/kysec_fpro/test_fpro.sh
 # @Description: test scripts for kylin security file protect
 #
 # Copyright (C) 2022-2024 KylinSoft Corporation.
###

###
 # @function: test_fpro_on
 # @description: kysec file protect enable test
 # @return {*} 0: success; !0: failed
###
function test_fpro_on {
    setstatus -f fpro on
    start_time=`date +%s`
    file=test-${start_time}
    touch /tmp/$file
    kysec_set -n protect -v readonly  /tmp/$file
    echo "hello" >> /tmp/$file
    ret_readonly_write=$?

    mv  /tmp/$file /tmp/$file-1
    ret_readonly_mv=$?

    rm -rf  /tmp/$file
    ret_readonly_rm=$?

    kysec_set -xn protect  /tmp/$file
    echo "hello" >> /tmp/$file
    ret_none_write=$?

    rm -rf  /tmp/$file
    ret_none_rm=$?

    if [ $ret_readonly_write -eq 1 ] && [ $ret_readonly_mv -eq 1 ] && [ $ret_readonly_rm -eq 1 ] \
	    && [ $ret_none_write -eq 0 ] && [ $ret_none_rm -eq 0 ]
    then
    	echo "fpro  test ok......"
    	ret_fro_on=0
    	echo `date +%Y%m%d%H%M%S`:test_fpro:test_fpro_on:res ok >>   res_fpro
    else
    	echo "fpro test failed......"
    	ret_fpro_on=-1
    	echo `date +%Y%m%d%H%M%S`:test_fpro:test_fpro_on:res failed >> res_fpro
    fi
    return $ret_fpro_on
}

###
 # @function: test_fpro_off
 # @description: kysec file protect disable test
 # @return {*} 0: success; !0: failed
###
function test_fpro_off {
    setstatus -f fpro off
    start_time=`date +%s`
    file=test-${start_time}
    touch /tmp/$file
    kysec_set -n protect -v readonly  /tmp/$file
    echo "hello" >> /tmp/$file
    ret_off_write=$?

    rm -rf  /tmp/$file
    ret_off_rm=$?

    if [ $ret_off_write -eq 0 ] && [ $ret_off_rm -eq 0 ]
    then
    	echo "fpro  test ok......"
    	ret_fro_off=0
    	echo `date +%Y%m%d%H%M%S`:test_fpro:test_fpro_off:res ok >>   res_fpro
    else
    	echo "fpro test failed......"
    	ret_fpro_off=-1
	    echo `date +%Y%m%d%H%M%S`:test_fpro:test_fpro_off:res failed >> res_fpro
    fi
    return $ret_fpro_off
}

rm -rf res_fpro
test_fpro_on
res_test_fpro_on=$?
sleep 2
test_fpro_off
res_test_fpro_off=$?


if [ $res_test_fpro_on -eq 0 ] && [ $res_test_fpro_off -eq 0 ]
then
    echo `date +%Y%m%d%H%M%S`:test_fpro_case:testcase:res ok >> res_fpro
else
    echo `date +%Y%m%d%H%M%S`:test_fpro_case:testcase:res failed >> res_fpro
fi
