#!/bin/bash
# SPDX-License-Identifier: GPL-2.0
###
 # @Author: zhubing <zhubing@kylinos.cn>
 # @FilePath: /kysec_testcase/kysec_fpro/test_kysec_fpro.sh
 # @Description: test scripts for kylin security file protect
 #
 # Copyright (C) 2022-2024 KylinSoft Corporation.
###

###
 # @function: test_kysec_fpro_on
 # @description: kysec file protect enable test
 # @return {*} 0: success; !0: failed
###
function test_kysec_fpro_on {
    setstatus -f fpro on
    start_time=`date +%s`
    file=test-${start_time}
    touch /tmp/$file
    kysec_set -n protect -v readonly  /tmp/$file
    echo "hello" >> /tmp/$file
    ret_readonly_write=$?

    mv  /tmp/$file /tmp/$file-1
    ret_readonly_mv=$?

    rm -rf  /tmp/$file
    ret_readonly_rm=$?

    kysec_set -xn protect  /tmp/$file
    echo "hello" >> /tmp/$file
    ret_none_write=$?

    rm -rf  /tmp/$file
    ret_none_rm=$?

    if [ $ret_readonly_write -eq 1 ] && [ $ret_readonly_mv -eq 1 ] && [ $ret_readonly_rm -eq 1 ] \
	    && [ $ret_none_write -eq 0 ] && [ $ret_none_rm -eq 0 ]
    then
    	echo "fpro  test ok......"
    	ret_fro_on=0
    	echo `date +%Y%m%d%H%M%S`:test_kysec_fpro:test_kysec_fpro_on:res ok >>   res_kysec_fpro
    else
    	echo "fpro test failed......"
    	ret_fpro_on=-1
    	echo `date +%Y%m%d%H%M%S`:test_kysec_fpro:test_kysec_fpro_on:res failed >> res_kysec_fpro
    fi
    return $ret_fpro_on
}

###
 # @function: test_kysec_fpro_off
 # @description: kysec file protect disable test
 # @return {*} 0: success; !0: failed
###
function test_kysec_fpro_off {
    setstatus -f fpro off
    start_time=`date +%s`
    file=test-${start_time}
    touch /tmp/$file
    kysec_set -n protect -v readonly  /tmp/$file
    echo "hello" >> /tmp/$file
    ret_off_write=$?

    rm -rf  /tmp/$file
    ret_off_rm=$?

    if [ $ret_off_write -eq 0 ] && [ $ret_off_rm -eq 0 ]
    then
    	echo "fpro  test ok......"
    	ret_fro_off=0
    	echo `date +%Y%m%d%H%M%S`:test_kysec_fpro:test_kysec_fpro_off:res ok >>   res_kysec_fpro
    else
    	echo "fpro test failed......"
    	ret_fpro_off=-1
	    echo `date +%Y%m%d%H%M%S`:test_kysec_fpro:test_kysec_fpro_off:res failed >> res_kysec_fpro
    fi
    return $ret_fpro_off
}

###
 # @function: test_kysec_fpro_kysoft
 # @return {*} 0: success; !0: failed
###
function test_kysec_fpro_kysoft {
    setstatus -f fpro on
    start_time=`date +%s`
    file=test-${start_time}
    touch /tmp/$file
    kysec_set -n protect -v readonly  /tmp/$file
    cp /bin/rm /tmp/rm_fprotest
    kysec_set -n exectl -v kysoft /tmp/rm_fprotest

    /tmp/rm_fprotest -rf  /tmp/$file
    ret=$?

    if [ $ret -eq 0 ]
    then
        echo "fpro  test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_fpro:test_kysec_fpro_kysoft:res ok >>   res_kysec_fpro
    else
        echo "fpro test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_fpro:test_kysec_fpro_kysoft:res failed >> res_kysec_fpro
    fi
    return $ret
}

###
 # @function: test_kysec_fpro_parent
 # @return {*} 0: success; !0: failed
###
function test_kysec_fpro_parent {
    setstatus -f fpro on
    start_time=`date +%s`
    file=test-${start_time}
    touch /tmp/$file
    kysec_set -n protect -v readonly  /tmp/$file
    cp /bin/rm /tmp/rm_fprotest
    kysec_set -n exectl -v parent /tmp/rm_fprotest

    /tmp/rm_fprotest -rf  /tmp/$file
    ret=$?

    if [ $ret -eq 0 ]
    then
        echo "fpro  test ok......"
        ret=0
        echo `date +%Y%m%d%H%M%S`:test_kysec_fpro:test_kysec_fpro_parent:res ok >>   res_kysec_fpro
    else
        echo "fpro test failed......"
        ret=-1
        echo `date +%Y%m%d%H%M%S`:test_kysec_fpro:test_kysec_fpro_parent:res failed >> res_kysec_fpro
    fi
    return $ret
}


rm -rf res_kysec_fpro
kysec_status=`cat /sys/kernel/security/kysec/status`
if [ $kysec_status -eq 2 ] ; then
    echo "kysec is enable"
elif [ $kysec_status -eq 4 ] ; then
    echo "kysec status is softmode"
    setstatus enable
else
    echo `date +%Y%m%d%H%M%S`:kysec_fpro:testcase:res failed >> res_kysec_fpro
    echo `date +%Y%m%d%H%M%S`:"kysec staus is disable" >> res_kysec_fpro
    exit
fi


test_kysec_fpro_on
res_test_kysec_fpro_on=$?
sleep 1
test_kysec_fpro_off
res_test_kysec_fpro_off=$?
sleep 1
test_kysec_fpro_kysoft
res_test_kysec_fpro_kysoft=$?
sleep 1
test_kysec_fpro_parent
res_test_kysec_fpro_parent=$?

if [ $res_test_kysec_fpro_on -eq 0 ] && [ $res_test_kysec_fpro_off -eq 0 ] && \
	[ $res_test_kysec_fpro_kysoft -eq 0 ] && [ $res_test_kysec_fpro_parent -eq 0 ]
then
    echo `date +%Y%m%d%H%M%S`:test_kysec_fpro_case:testcase:res ok >> res_kysec_fpro
else
    echo `date +%Y%m%d%H%M%S`:test_kysec_fpro_case:testcase:res failed >> res_kysec_fpro
fi

setstatus softmode
