#!/bin/bash
# SPDX-License-Identifier: GPL-2.0
###
 # @Author: zhubing <zhubing@kylinos.cn>
 # @FilePath: /kysec_testcase/kysec_kmod/test_kysec_kmod.sh
 # @Description: test scripts for kylin security kernel module protect
 #
 # Copyright (C) 2022-2024 KylinSoft Corporation.
###

###
 # @function: kysec_kmod_add
 # @description: add kmod to kmod protect testcase
 # @return {*} 0: success; !0: failed
###
function kysec_kmod_add {

    #echo "把nbd模块加入内核模块防卸载"
    ./kysec_kmod_demo add nbd
    sleep 2
    modprobe nbd
    ret=$?
    if [ $ret -ne 0 ]
    then
    	echo `date +%Y%m%d%H%M%S`:kysec_kmod:testcase:res failed >> res_kysec_kmod
    	echo `date +%Y%m%d%H%M%S`:"内核模块加载异常" >> res_kysec_kmod
    	exit
    fi
    sleep 2
    modprobe -r nbd
    ret=$?
    if [ $ret -eq 1 ]
    then
    #	echo "卸载内核模块成功"
    	echo `date +%Y%m%d%H%M%S`:kysec_kmod:kysec_kmod_add:res ok >>   res_kysec_kmod
    	res=0
    else
    #	echo "卸载内核模块失败"
    	echo `date +%Y%m%d%H%M%S`:kysec_kmod:kysec_kmod_add:res failed >>   res_kysec_kmod
    	res=-1
    fi

    return $res
}

###
 # @function: kysec_kmod_remove
 # @description: remove kmod from kmod_protect testcase
 # @return {*} 0: success; !0: failed
###
function kysec_kmod_remove {

    #echo "把nbd模块从内核模块防卸载中移除"
    ./kysec_kmod_demo remove nbd
    sleep 2
    modprobe -r nbd
    ret=$?
    if [ $ret -eq 0 ]
    then
    #	echo "卸载内核模块成功"
    	echo `date +%Y%m%d%H%M%S`:kysec_kmod:kysec_kmod_remove:res ok >>   res_kysec_kmod
    	res=0
    else
    #	echo "卸载内核模块失败"
    	echo `date +%Y%m%d%H%M%S`:kysec_kmod:kysec_kmod_remove:res failed >>   res_kysec_kmod
    	res=-1
    fi

    return $ret
}

rm -rf res_kysec_kmod
kysec_status=`cat /sys/kernel/security/kysec/status`
if [ $kysec_status -eq 2 ] ; then
    echo "kysec is enable"
elif [ $kysec_status -eq 4 ] ; then
    echo "kysec status is softmode"
    setstatus enable
else
    echo `date +%Y%m%d%H%M%S`:kysec_kmod:testcase:res failed >> res_kysec_kmod
    echo `date +%Y%m%d%H%M%S`:"kysec staus is disable" >> res_kysec_kmod
    exit
fi

ppro_status=`cat /sys/kernel/security/kysec/kmod`
if [ $ppro_status -eq 0 ]
then
    setstatus -f kmod on
fi


path=`pwd`
filename="kysec_kmod_demo"
filepath=${path}/${filename}
echo $filename
# echo "给待测试程序授权"
./kysec_authorize_demo add kysec_kmod_demo $filepath 3

kysec_kmod_add
res_kysec_kmod_add=$?

kysec_kmod_remove
res_kysec_kmod_remove=$?

if [ $res_kysec_kmod_add -eq 0 ] && [ $res_kysec_kmod_remove -eq 0 ]
then
    echo `date +%Y%m%d%H%M%S`:kysec_kmod:testcase:res ok >> res_kysec_kmod
else
    echo `date +%Y%m%d%H%M%S`:kysec_kmod:testcase:res failed >> res_kysec_kmod
fi
setstatus softmode
