#!/bin/bash
# SPDX-License-Identifier: GPL-2.0
###
 # @Author: zhubing <zhubing@kylinos.cn>
 # @FilePath: /kysec_testcase/test_kysec_ppro/test_kysec_ppro.sh
 # @Description: test scripts for kylin security process protect
 #
 # Copyright (C) 2022-2024 KylinSoft Corporation.
###

###
 # @function: kysec_ppro_add_efi
 # @description: 将测试进程加入进程防杀列表
 # @return {*} 0: success; !0: failed
###
function kysec_ppro_add_efi {
    #echo "把/tmp/kysec_ppro_looprun加入进程防杀列表"
    ./kysec_ppro_demo add /tmp/kysec_ppro_looprun
    sleep 2
    nohup /tmp/kysec_ppro_looprun  >/dev/null &
    killall kysec_ppro_looprun
    ret=$?
    if [ $ret -eq 1 ]
    then
    #	echo "kill进程成功"
    	echo `date +%Y%m%d%H%M%S`:kysec_ppro:kysec_ppro_add_efi:res ok >>   res_kysec_ppro
    	res=0
    else
    #	echo "kill进程失败"
    	echo `date +%Y%m%d%H%M%S`:kysec_ppro:kysec_ppro_add_efi:res failed >>   res_kysec_ppro
    	res=-1
    fi

    return $res
}

###
 # @function: kysec_ppro_add_python
 # @description: 将python测试进程加入进程防杀列表
 # @return {*} 0: success; !0: failed
###
function kysec_ppro_add_python {
    #echo "把/tmp/kysec_ppro_looprun.py加入进程防杀列表"
    ./kysec_ppro_demo add /tmp/kysec_ppro_looprun.py
    sleep 2
    nohup /tmp/kysec_ppro_looprun.py  >/dev/null &
    killall kysec_ppro_looprun.py
    ret=$?
    if [ $ret -eq 1 ]
    then
    #   echo "kill进程成功"
        echo `date +%Y%m%d%H%M%S`:kysec_ppro:kysec_ppro_add_python:res ok >>   res_kysec_ppro
        res=0
    else
    #   echo "kill进程失败"
        echo `date +%Y%m%d%H%M%S`:kysec_ppro:kysec_ppro_add_python:res failed >>   res_kysec_ppro
        res=-1
    fi

    return $res
}

###
 # @function: kysec_ppro_remove_efi
 # @description: 将测试进程从进程防杀列表中移除
 # @return {*} 0: success; !0: failed
###
function kysec_ppro_remove_efi {
    ./kysec_ppro_demo remove /tmp/kysec_ppro_looprun
    sleep 2
    killall kysec_ppro_looprun
    ret=$?
    if [ $ret -eq 0 ]
    then
    #	echo "kill进程成功"
    	echo `date +%Y%m%d%H%M%S`:kysec_ppro:kysec_ppro_remove_efi:res ok >>   res_kysec_ppro
    	res=0
    else
    #	echo "kill进程失败"
    	echo `date +%Y%m%d%H%M%S`:kysec_ppro:kysec_ppro_remove_efi:res failed >>   res_kysec_ppro
    	res=-1
    fi

    return $res
}

###
 # @function: kysec_ppro_remove_python
 # @description: 将python测试进程从进程防杀列表中移除
 # @return {*} 0: success; !0: failed
###
function kysec_ppro_remove_python {
    ./kysec_ppro_demo remove /tmp/kysec_ppro_looprun.py
    sleep 2
    killall kysec_ppro_looprun.py
    ret=$?
    if [ $ret -eq 0 ]
    then
    #   echo "kill进程成功"
    	echo `date +%Y%m%d%H%M%S`:kysec_ppro:kysec_ppro_remove_python:res ok >>   res_kysec_ppro
    	res=0
    else
    #	echo "kill进程失败"
    	echo `date +%Y%m%d%H%M%S`:kysec_ppro:kysec_ppro_remove_python:res failed >>   res_kysec_ppro
    	res=-1
    fi

    return $res
}

rm -rf res_kysec_ppro
kysec_status=`cat /sys/kernel/security/kysec/status`
if [ $kysec_status -eq 2 ] ; then
	echo "kysec is enable"
elif [ $kysec_status -eq 4 ] ; then
	echo "kysec status is softmode"
	setstatus enable
else
	echo `date +%Y%m%d%H%M%S`:kysec_ppro:testcase:res failed >> res_kysec_ppro
	echo `date +%Y%m%d%H%M%S`:"kysec staus is disable" >> res_kysec_ppro
	exit
fi

ppro_status=`cat /sys/kernel/security/kysec/ppro`
if [ $ppro_status -eq 0 ]
then
	setstatus -f ppro on
fi

path=`pwd`
filename="kysec_ppro_demo"
filepath=${path}/${filename}
echo $filename
#echo "给待测试程序授权"
./kysec_authorize_demo add kysec_ppro_demo $filepath 2


kysec_ppro_add_efi
res_kysec_ppro_add_efi=$?

kysec_ppro_remove_efi
res_kysec_ppro_remove_efi=$?

kysec_ppro_add_python
res_kysec_ppro_add_python=$?

kysec_ppro_remove_python
res_kysec_ppro_remove_python=$?


if [ $res_kysec_ppro_add_efi -eq 0 ] && [ $res_kysec_ppro_remove_efi -eq 0 ] && \
	[ $res_kysec_ppro_add_python -eq 0 ] && [ $res_kysec_ppro_remove_python -eq 0 ]
then
    echo `date +%Y%m%d%H%M%S`:kysec_ppro:testcase:res ok >> res_kysec_ppro
else
    echo `date +%Y%m%d%H%M%S`:kysec_ppro:testcase:res failed >> res_kysec_ppro
fi

setstatus softmode
