# ./cve-cal.sh  <from-commit-id>
# used to calculate fixed cve bug since <from-commit-id>

. $(dirname $0)/init-functions

function usage()
{
	echo -e "`basename $0` <-t> <commit-id>"
	echo -e "  -h \t\t\tshow this help info"
}

ARGS=`getopt -o th -- "$@"`
if [ $? != 0 ]; then
	echo "Terminating..."
	usage && exit 1
fi

eval set -- "${ARGS}"

OMSG=""

while true
do
	case $1 in
		-h)
			usage
			exit 0;;
		--)
			shift
			break;;
	esac
done

last_release=$(git log --oneline  -1 CHANGELOG.md  | awk '{print $1}')

if ! [ -z $1 ]; then
	git log -1 $1 &> /dev/zero
	if [ $? -eq 0 ]; then
		last_release=$1
	else
		echo "commit id<$1> is not valid, use default $(git log --oneline -1 $last_release)"
	fi
fi

for id in $(git log ${last_release}.. --oneline | awk '{print $1}')
do
	task_id=`git show --no-patch $id | grep "[Tt]ask: \?\#\?[0-9]\{4,\}\|[Bb]ug: \?\#\?[0-9]\{4,\}" | sed -e 's/ //g' | sed -e 's/#//g' | sed -e 's/:/#/g'`;
	if [ x"$task_id" != x"" ]; then
		task_id="[`echo $task_id | sed 's/ /] [/g'`]"
	fi

	cve_id=`git show --no-patch $id | grep "CVE: \?CVE-[0-9]\{4,\}-[0-9]\{1,\}" | sed -e 's/CVE://g'`
	if [ x"${cve_id}" != x"" ]; then
		cve_id="`echo ${cve_id} | sed 's/ /, /g'`"
		trim "`printf -- "- %-14s %s" "${cve_id^^}" "${task_id,,}"`"
	fi
done | sort -u -k 2
